@@ -16,6 +16,9 @@
 
 (require 'x0213-cdef)
 
+(eval-when-compile
+  (require 'x0213-sjis))
+
 (if window-system
     (let (fontset)
       (when (setq fontset (query-fontset "-*-*-medium-r-*-*-16-*-*-*-*-*-*-*"))
@@ -35,12 +38,11 @@
 
 (define-ccl-program ccl-encode-shiftjis-font
   `(0
-    ((if (r1 % 2) (if (r2 <= 95) (r2 += 31) (r2 += 32)) (r2 += 126))
-     (if (r0 == ,(charset-id 'japanese-jisx0213-1))
-         (if (r1 <= 94) (r1 = ((r1 + 225) / 2)) (r1 = ((r1 + 353) / 2)))
-       (if (r1 <= 47)
-           ((r3 = ((r1 / 8) * 3)) (r1 = (((r1 + 471) / 2) - r3)))
-         (r1 = ((r1 + 379) / 2))))))
+    ((r4 = r1)
+     (r1 = (r4 en-sjis r2))
+     (r2 = r7)
+     (if (r0 == ,(charset-id 'japanese-jisx0213-2))
+	 ((r1 = r4 ,jisx0213-shift-jis-plain-2-encode-table)))))
   "CCL program to encode a JIS X 0213 to code point of Shift-JIS font.")
 
 (setq font-ccl-encoder-alist
