@@ -63,6 +63,17 @@
 	       0 (mucs-max-code)))
       sym)))
 
+(defun tae-project-all-arith-to-lambda ()
+  (if (get 'tae-project-all-arith-to-lambda
+	   'code-conversion-map)
+      'tae-project-all-arith-to-lambda
+    (register-code-conversion-map
+     'tae-project-all-special-to-identity
+     (vector t 'lambda
+	     (mucs-special-code 0)
+	     (1+ (mucs-special-code
+		  (1- mucs-code-range-specials)))))))
+
 (defun tae-project-special-to-identity-table (code)
   (let ((sym (tae-generate-project-table-symbol code)))
     (if (get sym 'code-conversion-map)
@@ -77,10 +88,10 @@
       'tae-project-all-special-to-identity
     (register-code-conversion-map
      'tae-project-all-special-to-identity
-     (vector t
+     (vector t t
 	     (mucs-special-code 0)
-	     (mucs-special-code
-	      (1- mucs-code-range-specials))))))
+	     (1+ (mucs-special-code
+		  (1- mucs-code-range-specials)))))))
 
 (defun tae-project-all-to-special-table (code)
   (tae-project-all-table (mucs-special-code code)))
@@ -298,9 +309,19 @@ which is managed by TAE."
 	     'tae-dependent-translations
 	     (cons dep-name trlist)))))
 
+(defun tae-get-dependent-translations (name)
+  (let ((dealing (list name))
+	dealt)
+    (while (setq name (car dealing))
+
+      (or (memq name dealt)
+	  (setq dealt (cons name dealt)))
+      (setq dealing (append (get name 'tae-dependent-translations)
+			    (cdr dealing))))
+    dealt))
+
 (defun tae-register-conversion (name type decodep conversion)
-  (let ((trlist (cons name
-		      (get name 'tae-dependent-translations)))
+  (let ((trlist (tae-get-dependent-translations name))
 	marksym
 	slot
 	convlist
@@ -341,21 +362,26 @@ after its compilation."
 
 ;;; Translation rule syntax
 ;;;
-;;; TR-ELEM := (`elisp' TYPE-SPEC Lisp_Func(num of args is 1) FUNC-RANGE [OPTION-ALIST]) 
-;;;            | (`assoc' TYPE-SPEC A-LIST [OPTION-ALIST])
+;;; TR-ELEM := (`assoc' TYPE-SPEC A-LIST [OPTION-ALIST])
+;;;            | (`range' TYPE-SPEC RANGE-A-LIST [OPTION-ALIST]) 
 ;;;            | (`ccl' TYPE-SPEC CCL_Program [OPTION-ALIST])
+;;; Notice that `ccl' TR-ELEM does not work currently.
 ;;;
 ;;; If type of TR-ELEM is A-LIST or CCL program,
 ;;;  this TR-ELEM is called PRECOMPILED.
 ;;;
-;;; FUNC-RANGE := DEF-RANGE [VAL-RANGE]
+;;; RANGE-A-LIST := (RANGE-A-LIST-ELEMENT ...)
+;;; RANGE-A-LIST-ELEMENT := (RANGE . RANGE) | (RANGE . OBJECT) | (OBJECT . RANGE)
 ;;; DEF-RANGE, VAL-RANGE := RANGE
 ;;; TYPE-SPEC := (CAR-TYPE . CDR-TYPE)
-;;; RANGE := ((RANGE1-MIN . RANGE1-MAX) (RANGE2-MIN . RANGE2-MAX) ... )
+;;; RANGE := ((OBJECT-MIN . OBJECT-MAX) ...)
+;;; A-LIST := ((OBJECT . OBJECT) ...)
+;;; OBJECT_MIN, OBJECT_MAX := OBJECT
 ;;;
 ;;; TYPE := registered type (see mucs-types.el)
+;;; OBJECT := Lisp Object
+;;;  (that mucs be valid in terms of the specified TYPE) (see mucs-types.el)
 ;;;
-;;; RANGEx-MIN, RANGEx-MAX := integer
 ;;;
 ;;; TRANSLATION := TRANSLATION-ELEMENT  |
 ;;;                DECLARED-TRANSLATION |
@@ -371,7 +397,7 @@ after its compilation."
 ;;;;;;   A-LIST note.
 ;;;;;;
 ;;;;;;     A-LIST is an association list to describe connections between
-;;;;;;   mainly NUMBER and NUMBER.
+;;;;;;   mainly OBJECT and OBJECT.
 ;;;;;;   For example, an A-LIST, ((A1 . A2) (B1 . B2) ... (Z1 . Z2)),
 ;;;;;;   means correspondences between A1 and A2, B1 and B2, and so on.
 ;;;;;;   The element of A-LIST is a cons cell.
@@ -389,6 +415,11 @@ after its compilation."
 ;;;;;;   that matches nothing and if a translated result is invalid, it
 ;;;;;;   ceases any further translation, and make the CCL state invalid.
 
+;;;;;;   RANGE-A-LIST note.
+;;;;;;     RANGE-A-LIST is NOT an association list in the normal meanings
+;;;;;;   of Lisp.  
+
+
 (defmacro tae-get-tr-elem-type (tr-elem-zzz)
   (list 'car tr-elem-zzz))
 
@@ -400,13 +431,13 @@ after its compilation."
   (list 'eq (list 'car-safe tr-elem-zzz)
 	''ccl))
 
-(defmacro tae-tr-elem-elisp-p (tr-elem-zzz)
+(defmacro tae-tr-elem-range-p (tr-elem-zzz)
   (list 'eq (list 'car-safe tr-elem-zzz)
-	''elisp))
+	''range))
 
 (defmacro tae-tr-elem-p (tr-elem-zzz)
   (list 'memq (list 'car-safe tr-elem-zzz)
-	''(assoc elisp ccl)))
+	''(assoc range ccl)))
 
 (defsubst tae-get-tr-elem-type-spec (tr-elem)
   (nth 1 tr-elem))
@@ -420,29 +451,29 @@ after its compilation."
 (defsubst tae-get-tr-elem-ccl (tr-elem)
   (nth 2 tr-elem))
 
-(defsubst tae-get-tr-elem-elisp-func (tr-elem)
+(defsubst tae-get-tr-elem-range-alist (tr-elem)
   (nth 2 tr-elem))
 
-(defsubst tae-get-tr-elem-range (tr-elem)
-  (if (tae-tr-elem-elisp-p tr-elem)
-      (nth 3 tr-elem)
-    (error "This translation element:%S have no RANGE."
-	   tr-elem)))
+;(defsubst tae-get-tr-elem-range (tr-elem)
+;  (if (tae-tr-elem-elisp-p tr-elem)
+;      (nth 3 tr-elem)
+;    (error "This translation element:%S have no RANGE."
+;	   tr-elem)))
 
 (defsubst tae-get-tr-elem-option-alist (tr-elem)
-  (cond ((tae-tr-elem-assoc-p tr-elem)
-	 (nth 3 tr-elem))
-	((tae-tr-elem-elisp-p tr-elem)
-	 (nth 4 tr-elem))
-	((tae-tr-elem-ccl-p tr-elem)
-	 (nth 3 tr-elem))))
+   (cond ((tae-tr-elem-assoc-p tr-elem)
+ 	 (nth 3 tr-elem))
+ 	((tae-tr-elem-range-p tr-elem)
+ 	 (nth 3 tr-elem))
+ 	((tae-tr-elem-ccl-p tr-elem)
+ 	 (nth 3 tr-elem))))
 
 (defsubst tae-set-tr-elem-option-alist (tr-elem alist)
   (let (slot)
     (cond ((tae-tr-elem-assoc-p tr-elem)
 	   (setq slot (nthcdr 3 tr-elem)))
-	  ((tae-tr-elem-elisp-p tr-elem)
-	   (setq slot (nthcdr 4 tr-elem)))
+	  ((tae-tr-elem-range-p tr-elem)
+	   (setq slot (nthcdr 3 tr-elem)))
 	  ((tae-tr-elem-ccl-p tr-elem)
 	   (setq slot (nthcdr 3 tr-elem))))
     (if slot
@@ -618,39 +649,15 @@ When decodep is non-nil, normalize for decording."
 					  (cdr all-slot))))
 	   ))
 
-	((tae-tr-elem-elisp-p tr-elem)
-	 (let ((range-list (tae-check-and-normalize-range
-			    (tae-get-tr-elem-range tr-elem)))
-	       (func (tae-get-tr-elem-elisp-func tr-elem))
-	       range-elem
-	       i j
-	       alist)
-	   (while (setq range-elem (car range-list))
-	     (cond ((numberp range-elem)
-		    (setq alist (cons
-				 (cons range-elem
-				       (funcall func range-elem))
-				 alist)))
-		   ((consp range-elem)
-		    (setq i (car range-elem)
-			  j (cdr range-elem))
-		    (while (<= i j)
-		      (setq alist (cons
-				   (cons i
-					 (funcall func i))
-				   alist)
-			    i (1+ i))))
-		   (t
-		    (error "Invalid range:%S" range-elem)))
-	     (setq range-list (cdr range-list)))
-	   (setq tr-elem
-		 (list 'assoc
-		       (tae-get-tr-elem-type-spec tr-elem)
-		       (nreverse alist)))))
+	((tae-tr-elem-range-p tr-elem)
+	 (let ((alist (tae-get-tr-elem-range-alist tr-elem))
+	       range-elem i j
+	       newalist)
+	   ;;; We should implement normalization process on range tr-elem.
+	   ))
 
 	((tae-tr-elem-ccl-p tr-elem)
-	 )
-	)
+	 ))
   (tae-tr-elem-set-normalized-flag tr-elem decodep)
   tr-elem)
 
@@ -729,10 +736,10 @@ When DECODEP is t, reduce for decodeing translation."
 
 		     result))
 
-		  ((and (or (tae-tr-elem-ccl-p tr1)
-			    (tae-tr-elem-elisp-p tr1))
-			(or (tae-tr-elem-ccl-p tr2)
-			    (tae-tr-elem-elisp-p tr2)))
+		  ((or (or (tae-tr-elem-ccl-p tr1)
+			   (tae-tr-elem-range-p tr1))
+		       (or (tae-tr-elem-ccl-p tr2)
+			   (tae-tr-elem-range-p tr2)))
 		   nil)
 		  ((or (tae-declared-translation-p tr1)
 		       (tae-declared-translation-p tr2))
@@ -802,10 +809,10 @@ When DECODEP is t, reduce for decodeing translation."
 		       (list 'assoc
 			     (tae-get-tr-elem-type-spec tr1)
 			     result-alist)))
-		    ((and (or (tae-tr-elem-ccl-p tr1)
-			      (tae-tr-elem-elisp-p tr1))
-			  (or (tae-tr-elem-ccl-p tr2)
-			      (tae-tr-elem-elisp-p tr2)))
+		    ((or (or (tae-tr-elem-ccl-p tr1)
+			     (tae-tr-elem-range-p tr1))
+			 (or (tae-tr-elem-ccl-p tr2)
+			     (tae-tr-elem-range-p tr2)))
 		     nil)
 		    ((or (tae-declared-translation-p tr1)
 			 (tae-declared-translation-p tr2))
@@ -917,10 +924,10 @@ When DECODEP is t, reduce for decodeing translation."
 			   result-alist
 			   (list (cons 'all tr-new-all-key)))
 		     ))
-		  ((and (or (tae-tr-elem-ccl-p tr1)
-			    (tae-tr-elem-elisp-p tr1))
-			(or (tae-tr-elem-ccl-p tr2)
-			    (tae-tr-elem-elisp-p tr2)))
+		  ((or (or (tae-tr-elem-ccl-p tr1)
+			   (tae-tr-elem-range-p tr1))
+		       (or (tae-tr-elem-ccl-p tr2)
+			   (tae-tr-elem-range-p tr2)))
 		   nil)
 		  ((or (tae-declared-translation-p tr1)
 		       (tae-declared-translation-p tr2))
@@ -1104,6 +1111,17 @@ according to it, rearrange the elements of UNIT."
 	  message)
       (cons (cons 'new-type type) message))))
 
+(defsubst tae-message-set-arithmetic (message flag)
+  (let ((slot (assq 'arithmetic message)))
+    (if slot
+	(progn
+	  (setcdr slot flag)
+	  message)
+      (cons (cons 'arithmetic flag) message))))
+
+(defsubst tae-message-arithmetic-p (message)
+  (cdr (assq 'arithmetic message)))
+
 (defun tae-generate-new-table-set (tables name type decodep)
 ;  (message "NAME:%S TYPE:%S DECODEP:%S" name type decodep)
   (let (table-set)
@@ -1173,21 +1191,81 @@ according to it, rearrange the elements of UNIT."
 		((tae-tr-elem-ccl-p tr-elem)
 		 (list (cons 'ccl (tae-get-tr-elem-ccl tr-elem))))
 
-		((tae-tr-elem-elisp-p tr-elem)
-		 (error "TAE cannot compile Elisp translation element directly:%S"
-			tr-elem))))
+		((tae-tr-elem-range-p tr-elem)
+		 (if (not (tae-tr-elem-normalized-p tr-elem decodep))
+		     (error "TAE cannot compile unnormalized translation:%S"
+			    tr-elem))
+		 (let ((ralist (tae-get-tr-elem-range-alist tr-elem))
+		       slot1 slot2 st1 end1 st2 end2
+		       elem tables)
+
+		   (while (setq elem (car ralist))
+		     (setq ralist (cdr ralist))
+
+		     ;; set st1, st2, end1, end2
+		     (if decodep
+			 (setq slot1 (cdr elem)
+			       slot2 (car elem))
+		       (setq slot1 (car elem)
+			     slot2 (cdr elem)))
+		     (if (consp slot1)
+			 (setq st1
+			       (funcall type-func
+					(funcall conv-func
+						 (car slot1)))
+			       end1
+			       (funcall type-func
+					(funcall conv-func
+						 (cdr slot1))))
+		       (setq st1
+			     (funcall type-func
+				      (funcall conv-func
+					       slot1))
+			     end1 nil))
+		     (if (consp slot2)
+			 (setq st2
+			       (funcall to-func
+					(car slot2))
+			       end2
+			       (funcall to-func
+					(cdr slot2)))
+		       (setq st2
+			     (funcall to-func slot2)
+			     end2 nil))
+
+		     (cond ((and (null end1) (null end2))
+			    (error "Invalid range slot:%S!" elem))
+			   ((null end1)
+			    (setq end1 (+ st1 (- end2 st2))))
+			   ((null end2)
+			    (setq end2 (+ st2 (- end1 st1))))
+			   (t
+			    (if (/= (- end1 st1) (- end2 st2))
+				(error "Inconsistent range slot:%S(%d, %d)-(%d, %d)"
+				       elem st1 end1 st2 end2))))
+		     (setq tables
+			    (cons
+			     (vector t (+ (- st2 st1)
+					  (mucs-arithmetic-adjust))
+				     st1 (1+ end1))
+			     tables)))
+		   (list 
+		    (cons 'tables
+			  (nreverse tables))
+		    '(arithmetic . t))))))
 
     (tae-message-set-new-type result to-type)))
 
 (defun tae-compile-OR-operation (translations type decodep)
-  (let (tr mes-tbl mes tbls result-mes to-type new-type)
+  (let (tr mes-tbl mes tbls result-mes to-type new-type arith)
     (while (setq tr (car translations))
       (setq translations (cdr translations)
 	    mes-tbl (tae-get-message-from-translation
 		     tr type decodep)
 	    mes (car mes-tbl)
 	    tbls (nconc tbls (cdr mes-tbl))
-	    new-type (cdr (assq 'new-type mes)))
+	    new-type (cdr (assq 'new-type mes))
+	    arith (or arith (tae-message-arithmetic-p mes)))
 
       ;;; operate type.
       (if to-type
@@ -1199,11 +1277,15 @@ according to it, rearrange the elements of UNIT."
 
       ;; normal message keys are preserved.
       (setq result-mes (tae-message-append result-mes mes)))
-    (tae-message-set-tables result-mes
-			    tbls)))
+    (tae-message-set-arithmetic
+     (tae-message-set-tables result-mes
+			     tbls)
+     arith)))
 
 (defun tae-compile-AND-operation (translations type decodep)
-  (let (tr mes-tbl mes tbls result-mes to-type new-type)
+  (error "Sorry!. Compilation of AND operation is disabled intentionally.")
+  (let (tr mes-tbl mes tbls result-mes to-type new-type
+	   arith first-tbl)
     (while (setq tr (car translations))
       (setq translations (cdr translations)
 	    mes-tbl (tae-get-message-from-translation
@@ -1218,36 +1300,69 @@ according to it, rearrange the elements of UNIT."
 	(setq to-type new-type)
 	(setq result-mes
 	      (tae-message-set-new-type result-mes to-type)))
-
-      (setq tbls (nconc (list
-			 (list (nconc (cdr mes-tbl)
-				      (list (tae-project-all-to-special-table 1)))
-			       (tae-project-all-table t))
-			 (tae-project-special-to-identity-table 1))
-			tbls))
+      (if first-tbl
+	  (setq tbls (nconc (list
+			     (list (nconc (cdr mes-tbl)
+					  (list (tae-project-all-to-special-table 1)))
+				   (tae-project-all-table t))
+			     (tae-project-special-to-identity-table 1))
+			    tbls))
+	(setq first-tbl (cdr mes-tbl)
+	      arith (tae-message-arithmetic-p mes)))
 
       ;; normal message keys are preserved.
       (setq result-mes (tae-message-append result-mes mes)))
-    (tae-message-set-tables result-mes
-			    tbls)))
+    (setq tbls (list tbls first-tbl))
+    (tae-message-set-arithmetic
+     (tae-message-set-tables result-mes
+			     tbls)
+     arith)))
 
 (defun tae-compile-composite-operation (translations type decodep)
-  (let (tr mes-tbl mes tbls result-mes)
-    (while (setq tr (car translations))
-      (setq translations (cdr translations)
-	    mes-tbl (tae-get-message-from-translation
-		     tr type decodep)
-	    tbls (cons (cdr mes-tbl) tbls)
-	    mes (car mes-tbl)
-	    type (cdr (assq 'new-type mes)))
+  (error "Sorry!.  In this version, compilation of composite operation
+is disabled intentionally.")
+  (if decodep (setq translations (reverse translations)))
+  (let (tr mes-tbl mes tbls result-mes to-type new-type
+	   arith)
+    (setq tr (car translations)
+	  translations (cdr translations))
+    (if tr
+	(while 
+	    (progn
+	      (setq mes-tbl (tae-get-message-from-translation
+			     tr type decodep)
+		    mes (car mes-tbl)
+		    new-type (cdr (assq 'new-type mes))
+		    arith (tae-message-arithmetic-p mes))
 
-      ;; normal message keys are preserved.
-      (setq result-mes (tae-message-append result-mes mes)))
-    (tae-message-set-tables
-     (tae-message-set-new-type result-mes type)
-     (nreverse tbls))))
+	      ;; operate type.
+	      (if to-type
+		  (if (not (eq to-type new-type))
+		      (signal 'mucs-type-mismatch-error to-type new-type))
+		(setq to-type new-type)
+		(setq result-mes
+		      (tae-message-set-new-type result-mes to-type)))
+	      ;; normal message keys are preserved.
+	      (setq result-mes (tae-message-append result-mes mes))
+	      translations)
+	  (setq tbls (nconc tbls
+			    (list (nconc (cdr mes-tbl)
+					 (list (tae-project-all-to-special-table 1))))
+			    (if arith (list (tae-project-all-arith-to-lambda))))
+		tr (car translations)
+		translations (cdr translations))))
+
+    (setq tbls (nconc tbls
+		      (list (cdr mes-tbl)
+			    (list (tae-project-special-to-identity-table 1)))))
+    (tae-message-set-arithmetic
+     (tae-message-set-tables result-mes
+			     tbls)
+     arith)))
 
 (defun tae-compile-composite-transparent-operation (translations type decodep)
+  (error "Sorry!.  In this version, compilation of
+composite-transparent operation is disabled intentionally.")
   (if (/= (length translations) 2)
       (error "Composite Transparent requires two arguments:%S"
 	     translations))
@@ -1491,6 +1606,9 @@ according to it, rearrange the elements of UNIT."
       (cond ((eq 'new-type (car mes-el))
 	     (setq mucs-current-type (cdr mes-el)))
 
+	    ((eq 'arithmetic (car mes-el))
+	     ;; do nothing.
+	     )
 	    (t
 	     (error "Internal error.  unknown TAE message:%S" mes-el)
 	     )))
@@ -1511,17 +1629,20 @@ according to it, rearrange the elements of UNIT."
 (defun tae-compile-retrieve-ccl-prog (table-syms nestedp)
   (if nestedp
       `(loop
-	(r4 = r0)
 	(r1 = 0)
-	(map-multiple r1 r0 ,table-syms)
-	(if (r0 > ,(mucs-max-code))
-	    (r0 = ((r4 + r0) & ,(mucs-code-range-mask)))))
+	(loop
+	 (r4 = r0)
+	 (map-multiple r1 r0 ,table-syms)
+	 (if (r0 > ,(mucs-max-code))
+	     ((r0 += (r4 - ,(mucs-arithmetic-adjust)))
+	      (r1 += 1)
+	      (repeat)))))
     `(loop
-      (r4 = r0)
       (r1 = 0)
+      (r4 = r0)
       (iterate-multiple-map r1 r0 ,@table-syms)
       (if (r0 > ,(mucs-max-code))
-	  (r0 = ((r4 + r0) & ,(mucs-code-range-mask)))))))
+	  ((r0 += (r4 - ,(mucs-arithmetic-adjust))))))))
 
 ;; API for compilation.
 
@@ -1724,7 +1845,7 @@ a list<CDR> (TABLE<SYMBOL> TABLE<SYMBOL> ...)."
   (let ((translation-list (tae-registered-translation-rule-list))
 	tr result)
     (while (setq tr (car translation-list))
-;      (message "TR:%S EB:%S" tr (mucs-embedded-p 'tae-translation tr))
+      ;; (message "TR:%S EB:%S SP:%S" tr (mucs-embedded-p 'tae-translation tr)
       (setq result
 	    (append
 	     (tae-generate-persistent-program
